;; Copyright 2009, 2010, 2012, 2013, 2014, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; get-buffer-window can take nil in emacs24, but not xemacs21



;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

;;-----------------------------------------------------------------------------
;; run-hook-with-args

(defvar foo nil)
(add-to-list 'foo (lambda () (message "foo called")))
(run-hook-with-args 'foo)
(run-hook-with-args (symbol-value 'foo))



;;-----------------------------------------------------------------------------
;; forward-word

(require 'bytecomp)
(byte-compile-sexp '(defun foo () (forward-word 1)))
(byte-compile-sexp '(defun foo () (backward-word 1)))
(byte-compile-sexp '(defun foo () (forward-word -1)))


;;-----------------------------------------------------------------------------
;; switch-to-buffer

(switch-to-buffer nil)


;;-----------------------------------------------------------------------------
;; emacs21 noruntime warn

(progn
  (load "/home/gg/lisp/21/bytecomp.el")
  (byte-compile-file "/tmp/git.el"))

(progn
  (unload-feature 'bytecomp-simplify)
  (require 'bytecomp-simplify)
  (byte-compile-file "/tmp/git.el"))

(progn
  (setq message-log-max 1000)
  ;; (require 'apropos)
  (require 'bytecomp)

  (load "../bytecomp-simplify.el")
  (message "initial unresolved %S" byte-compile-unresolved-functions)

  (switch-to-buffer "*Messages*"))



;;-----------------------------------------------------------------------------
;; advice.el dependency

(progn
  ;; (load "advice")
  (add-to-list 'load-path "..")
  (load "bytecomp-simplify")
  (unload-feature 'advice)
  (unload-feature 'bytecomp-simplify))

(require 'advice
ad-find-advice
ad-advice-info
(symbol-function 'bytecomp-simplify-unload-function)

;;-----------------------------------------------------------------------------
(progn
  (require 'bytecomp)
  (load (expand-file-name "../bytecomp-simplify.el"))
  (byte-compile '(lambda () (char-before (point))))

  )

  (byte-compile '(lambda () (char-after (point))))
(macroexpand '(char-after (point)) byte-compile-macro-environment)
(macroexpand '(char-before (point)) byte-compile-macro-environment)
(get 'char-after 'byte-compile)
(get 'char-before 'byte-compile)
(get 'kill-buffer 'byte-compile)

;; (eval-buffer)
;; (setq debug-on-error t)
;; (byte-compile-file "align-let.el")

(prin1 zz (current-buffer))

(prin1 (get 'byte-compile-warnings 'custom-type)
       (current-buffer))

(assq 'x '(123 456))



;;-----------------------------------------------------------------------------
;; some byte compiles

(byte-compile-form '(put 'pointdir-with-default-directory 'lisp-indent-function 0))
(byte-compile-sexp '(defun foo () (kill-buffer (current-buffer))))

(byte-compile-sexp '(progn
                      (defmacro t-bytecomp-simplify--macro4 ()
                        '(message "hi"))))


;;-----------------------------------------------------------------------------
;; (declare (indent 1))

(require 'bytecomp)
(fmakunbound 'foo)
(byte-compile-sexp '(progn
                      (defmacro xxfoo () '(message "hi"))
                      (put 'xxfoo 'lisp-indent-function 1)
                      (defun bar () (foo) quux)))
(apropos-macrop 'foo)

(byte-compile-sexp '(defun bar () (foo)))

(defun foo ()
  (declare (indent 2))
  nil)
(symbol-function 'foo)
(functionp 'foo)
(put 'foo 'lisp-indent-function 2)
(foo
    1
    2
  3)

(defmacro bar ()
  (declare (indent 1))
  '(message "hi"))
(bar
    1
  2
  3)
(autoload 'apropos-macrop "apropos")
(apropos-macrop 'bar)


;;-----------------------------------------------------------------------------
;; emacs cvs

(progn
  (require 'bytecomp)
  (setq byte-compile-warnings '(simplify))
  (compile
   "mkdir /tmp/bytecomp-simplify; cp /down/emacs/lisp/*.el /down/emacs/lisp/*/*.el /tmp/bytecomp-simplify;
   "))

;;     emacs "/tmp/bytecomp-simplify/*.el"))
;;     (byte-compile-file filename)))

(compile (concat "cd /tmp/bytecomp-simplify/;"
                 " emacs-cvs-b -Q -batch"
                 " --eval \"(setq byte-compile-warnings '(simplify))\""
                 " -l " (expand-file-name "../bytecomp-simplify.el")
                 " -f batch-byte-compile *.el"))

;;-----------------------------------------------------------------------------
;; byte-compile-simplify-regexp-fixed-p

(string-match "[\\]]" "\\]")
(string-match "[\n]" "\n")
(string-match "[^]]" "\n")
(string-match "[^^]" "\n")
(byte-compile-simplify-regexp-fixed-p "[^]]")

;;-----------------------------------------------------------------------------

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))

(progn
  (elint-initialize)
  (find-file "bytecomp-simplify.el")
  (elint-current-buffer))


;;-----------------------------------------------------------------------------
;; princ

(princ "\n" (current-buffer))
(byte-compile '(lambda () (princ "\n")))
(progn
  (defun foo () (princ "\n" (current-buffer)))
  (byte-compile 'foo)
  (disassemble 'foo))

;; newlines
(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH '(princ \"\\\\n\"'")

;; empties
(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH '(princ \"\"'")


(prin1 "\n" (current-buffer))
(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH '(print \"'")
(print "x" (current-buffer))
(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH 'prin.*standard-output)'")

(let ((standard-output (current-buffer)))
  (princ-list "x" "y" ?z))

;;-----------------------------------------------------------------------------
;; (eq nil ...)

(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH '(eq nil'")

(progn
  (defun foo (obj) (eq nil obj))
  (byte-compile 'foo)
  (disassemble 'foo))
(byte-compile-sexp '(lambda (x) (eq x nil)))
(byte-compile-sexp '(lambda (x) (equal x nil)))
(byte-compile-sexp '(lambda (x) (eql x nil)))

;;-----------------------------------------------------------------------------
;; (equal 'foo ...)

(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH '(equal \'[a-zA-Z-]'")

(progn
  (defun foo (obj) (eql 123.5 obj))
  (byte-compile 'foo)
  (disassemble 'foo))
(byte-compile-sexp '(lambda (x) (equal 'foo x)))
(byte-compile-sexp '(lambda (x) (equalp 'foo x)))
(byte-compile-sexp '(lambda (x) (equal-including-properties 'foo x)))
(byte-compile-sexp '(lambda (x) (eql nil x)))

(pp (symbol-plist 'eql) (current-buffer))
(get 'eql 'byte-compile)
(get 'eql 'byte-optimize)
(disassemble 'cl-byte-compile-compiler-macro)


;;-----------------------------------------------------------------------------
;; cc-mode ?

(debug-on-entry 'display-warning)
(cancel-debug-on-entry 'display-warning)
(find-file "/tmp/x.c")
