;; Copyright 2006, 2007, 2009, 2010, 2011, 2013, 2015 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-align-let--temp*"))
    (switch-to-buffer tempbuf)
    (emacs-lisp-mode)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(align-let))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((emacs-lisp-mode-hook  align-let-keybinding)
              (scheme-mode-hook      align-let-keybinding)))

    (unless (get 'align-let-spaces 'safe-local-variable)
      (error "align-let-spaces 'safe-local-variable not autoloaded"))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; safe-local-variable-p

    (if (not (fboundp 'safe-local-variable-p))
        (message "skip, no `safe-local-variable-p'")

      (unless (safe-local-variable-p 'align-let-spaces 3)
        (error "align-let-spaces=3 not safe-local-variable-p"))
      (setq count (1+ count))

      (when (safe-local-variable-p 'align-let-spaces "blah")
        (error "align-let-spaces=string is safe-local-variable-p"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'align-let)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; align-let-save-point

    (require 'align-let)
    (if (not (fboundp 'align-let-save-point))
        (message "skip, no `align-let-save-point' macro after byte compiled")

      (let ((want 123)
            (got  (align-let-save-point 123)))
        (unless (equal got want)
          (error "align-let-save-point\ngot:  %s\nwant: %s\nat count %s"
                 got want count)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; align-let-backward-to-code

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (pos  (nth 1 elem))
                    (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (align-let-backward-to-code)
                (let ((got (- (point) (point-min))))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (erase-buffer)
                    (insert "\n---str---\n")
                    (insert str)
                    (insert (format "\n\nalign-let-backward-to-code\ngot:  %s\nwant: %s\nat count %s"
                                    got want count))
                    (error "align-let-backward-to-code\ngot:  %s\nwant: %s\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(("" 0 0)
              ("\"" 0 0)
              ("\"" 1 0)))


    ;;-------------------------------------------------------------------------
    ;; align-let-looking-at-nest

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (not (not (align-let-looking-at-nest)))))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (insert "\n---str---\n")
                    (insert str)
                    (insert (format "\n\nalign-let-looking-at-nest\ngot:  %s\nwant: %s\nat count %s"
                                    got want count))
                    (error "align-let-looking-at-nest\ngot:  %s\nwant: %s\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(("" nil)
              ("(" nil)

              ("((" t)
              ("( (" t)
              ("\n(\n\t(" t)
              ("\n(\n;;foo\n;;bar\n\n(" t)

              ("[[" t)))


    ;;-------------------------------------------------------------------------
    ;; align-let-looking-at-let

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (not (not (align-let-looking-at-let)))))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (insert "\n---str---\n")
                    (insert str)
                    (insert (format "\n\nalign-let-looking-at-let\ngot:  %s\nwant: %s\nat count %s"
                                    got want count))
                    (error "got:  %s\nwant: %s\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(("(" nil)
              ("(let ((x 1)))" t)
              ("(let" nil)
              ("(let (x" nil)
              ("(let ((x" t)

              ("(let* ((x" t)
              ("(lexical-let  ((x" t) ;; from 'cl
              ("(lexical-let* ((x" t) ;; from 'cl

              ("(and-let* (x))" t)
              ("(and-let* ((x 1)))" t)

              ("(setq x 1 y 2)" t)
              ("(setq x 1 y 2" t)
              ("(setq x 1)"  nil)
              ("(setq x 1"   nil)
              ("(setq x 1 y" nil)))


    ;;-------------------------------------------------------------------------
    ;; align-let

    (mapcar (lambda (elem)
              (let ((input (nth 0 elem))
                    (want  (nth 1 elem)))
                (erase-buffer)
                (insert input)
                (goto-char (point-min))
                (end-of-line)
                (let ((got (condition-case err
                               (progn
                                 (align-let)
                                 (buffer-string))
                             (error (format "error: %s"
                                            (error-message-string err))))))
                  (unless (equal want got)
                    (goto-char (point-max))
                    (erase-buffer)
                    (insert "\n---input---\n")
                    (insert input)
                    (insert "\n---got---\n")
                    (insert (format "%s\n" got))
                    (insert "\n---want---\n")
                    (insert want)
                    (error "wrong at %s\n%s\n" count (buffer-string)))))
              (setq count (1+ count)))

            '(

;;; ---------------------
;;; unclosed `let', but closed bindings form
              ("\
 (let ((x 1)
       (yy 2))
" "\
 (let ((x  1)
       (yy 2))
")

;;; ---------------------
;;; unclosed bindings form
              ("\
 (let ((x 1)
       (yy 2)" "\
 (let ((x  1)
       (yy 2)")

;;; ---------------------
;;; final stray (
              ("\
 (let ((x 1)
       (yy 2)
       (" "\
 (let ((x  1)
       (yy 2)
       (")

;;; ---------------------
;;; incomplete binding
              ("\
 (let ((x 1)
       (yy 2)
       (zzz" "\
 (let ((x  1)
       (yy 2)
       (zzz")

;;; ---------------------
;;; empty doesn't look like a let
              ("\
 (let (
      )
   nil)
" "error: Not in a \"let\" form")

;;; ---------------------
;;; with only no-value vars doesn't look like a let
              ("\
 (let (a b c)
   nil)
" "error: Not in a \"let\" form")

;;; ---------------------
;;; one on single line
              ("\
 (let ((foo  1) a b c )
   nil)
" "\
 (let ((foo 1) a b c )
   nil)
")

;;; ---------------------
;;; extra before ((
              ("\
 (let-foo bar ((foo  1))
   nil)
" "\
 (let-foo bar ((foo 1))
   nil)
")

;;; ---------------------
;;; extra before ((
              ;; not yet ...
              ;;             ("\
              ;;  (let-foo bar quux ((foo  1))
              ;;    nil)
              ;; " "\
              ;;  (let-foo bar quux ((foo 1))
              ;;    nil)
              ;; ")

;;; ---------------------
;;; long var, value not on its line
              ("\
 (let ((quux
        3)
       (foo 1)
       (bar        2))
   nil)
"

               "\
 (let ((quux
        3)
       (foo 1)
       (bar 2))
   nil)
"
               ;; but going to be ...
               ;; "\
               ;;  (let ((quux
               ;;         3)
               ;;        (foo 1)
               ;;        (bar 2))
               ;;    nil)
               ;; "
               )

;;; ---------------------
;;; not and-let*
              ("\
 (and-lt* (x
   (x 1)
   (xy 2))
" "error: Not in a \"let\" form")

;;; ---------------------
;;; and-let*
              ("\
  (and-let* (x
             (x 1)
             (xy 2))
" "\
  (and-let* (x
             (x  1)
             (xy 2))
")

;;; ---------------------
;;; long var without value
              ("\
 (let ((foo 1)
       (bar        2)
       quuxxxxxxx)
   nil)
" "\
 (let ((foo 1)
       (bar 2)
       quuxxxxxxx)
   nil)
")

;;; ---------------------
;;; long var without value, both others change
              ("\
 (let ((foo   1)
       (bar        2)
       quuxxxxxxx)
   nil)
" "\
 (let ((foo 1)
       (bar 2)
       quuxxxxxxx)
   nil)
")

;;; ---------------------
;;; with comments
              ("\
 (let ((foo   1) ;; hello
       (bar        2) ;; world

       )
   nil)
" "\
 (let ((foo 1) ;; hello
       (bar 2) ;; world

       )
   nil)
")

;;; ---------------------
;;; comment after last
              ("\
 (let ((foo   1) ;; hello
       (bar        2) ;; world
 ;; blah
       )
   nil)
" "\
 (let ((foo 1) ;; hello
       (bar 2) ;; world
 ;; blah
       )
   nil)
")))


    ;;-------------------------------------------------------------------------
    ;; align-let-region

    (mapcar (lambda (elem)
              (let ((input (nth 0 elem))
                    (want  (nth 1 elem)))
                (erase-buffer)
                (insert input)
                (goto-char (point-min))
                (end-of-line)
                (let ((got (condition-case err
                               (progn
                                 (align-let-region (point-min) (point-max))
                                 (buffer-string))
                             (error (format "error: %s"
                                            (error-message-string err))))))
                  (unless (equal want got)
                    (goto-char (point-max))
                    (erase-buffer)
                    (insert "\n\nalign-let-region\n---input---\n")
                    (insert input)
                    (insert "\n---got---\n")
                    (insert (format "%s\n" got))
                    (insert "\n---want---\n")
                    (insert want)
                    (error "align-let-region wrong at %s\n%s\n"
                           count (buffer-string)))))
              (setq count (1+ count)))

            '(

;;; ---------------------
;;; two forms
              ("\
 (let ((x 1)
                  (yy 2))
  (+ x y))
  (let ((x 1)
        (yy  9)
" "\
 (let ((x  1)
                  (yy 2))
  (+ x y))
  (let ((x  1)
        (yy 9)
")

;;; ---------------------
;;; nested in value
              ("\
 (let ((x 1)
       (yy  (let ((x 1)
                  (yy 2))))
" "\
 (let ((x  1)
       (yy (let ((x  1)
                  (yy 2))))
")
              ))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'align-let)
    (unload-feature 'align-let t)
    (require 'align-let)
    (unload-feature 'align-let t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
