;; Copyright 2010, 2011 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defconst my-perl-available-p
  (condition-case nil
      (equal 0 (call-process "perl"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"perl\" program is available.")

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((case-fold-search nil)
        (count 0)
        (tempbuf "*t-ffap-perl-module--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap-perl-module)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))

    (message "ffap-perl-module-directory-regexp is %S"
             ffap-perl-module-directory-regexp)

    ;;-------------------------------------------------------------------------
    ;; ffap-perl-module-directory-regexp

    (when (string-match ffap-perl-module-directory-regexp ".")
      (error "ffap-perl-module-directory-regexp should not match \".\"")
      (setq count (1+ count)))

    (when (string-match ffap-perl-module-directory-regexp "..")
      (error "ffap-perl-module-directory-regexp should not match \"..\"")
      (setq count (1+ count)))

    (when (string-match ffap-perl-module-directory-regexp "Foo.pm")
      (error "ffap-perl-module-directory-regexp should not match \"Foo.pm\"")
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; ffap-perl-module-path

    (if (not my-perl-available-p)
        (message "skip, perl not available")

      (setq ffap-perl-module-path nil)
      (let ((dirs (ffap-perl-module-path)))

        (unless (eq dirs ffap-perl-module-path)
          (error "ffap-perl-module-path return not the same as the variable"))
        (setq count (1+ count))

        (when (null dirs)
          (error "ffap-perl-module-path empty"))
        (setq count (1+ count))

        ;; make sure print and split-string are cooperating with "\n" separator
        (when (member "" dirs)
          (error "ffap-perl-module-path contains \"\""))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point advised

    (require 'ffap)
    (setq ffap-perl-module-path (list (expand-file-name ".")))

    ;; with {} braces included in 'file matching
    (let ((ffap-string-at-point-mode-alist
           (let ((elem (assq 'file ffap-string-at-point-mode-alist)))
             (setq elem (copy-sequence elem))
             (setcar (cdr elem) (concat (cadr elem) "{}"))
             (cons elem ffap-string-at-point-mode-alist))))

      (mapcar (lambda (elem)
                (let ((str  (nth 0 elem))
                      (want (nth 1 elem)))
                  (erase-buffer)
                  (insert str)
                  (goto-char (point-min))
                  (let ((got (ffap-string-at-point nil)))
                    (unless (equal got want)
                      (goto-char (point-max))
                      (erase-buffer)
                      (insert "\n---str---\n")
                      (insert str)
                      (insert (format "\n\nffap-string-at-point %s\ngot  %S\nwant %S\nat count %S\n"
                                      str got want count))
                      (error "ffap-string-at-point\ngot:  %s\nwant: %s\nat count %s"
                             got want count))))
                (setq count (1+ count)))

              '(("$NOSUCHVAR" "$NOSUCHVAR")
                ;; no error on unknown var
                ("$Foo::Bar{$NOSUCHVAR}" "$Foo::Bar{$NOSUCHVAR}")
                ("$" "$")
                ("${" "${"))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-perl-module))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-perl-module))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))

