;; Copyright 2006, 2007, 2009, 2010, 2012, 2014, 2017 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defvar my-have-env-on-command-p
  (require 'miscbits-el-loaddefs)
  (with-temp-buffer
    (call-process shell-file-name nil t nil
                  shell-command-switch
                  "LANG=C echo hello")
    (goto-char (point-min))
    (looking-at "hello")))

(defun t-man-completion--noop ()
  nil)
(defun t-man-completion--noop2 ()
  nil)

(defun my-run-test ()
  (let ((count 0)
        (tempbuf "*t-man-completion--temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)


    ;;------------------------------------------------------------------------
    ;; not autoloaded

    (require 'man)
    (when (featurep 'man-completion)
      (error "man-completion should not load with (require 'man)"))
    (setq count (1+ count))

    (require 'man-completion)

    (unless (ad-find-advice 'man 'before 'man-completion)
      (error "Missing advice on M-x man"))
    (setq count (1+ count))
    (unless (ad-find-advice 'manual-entry 'before 'man-completion)
      (error "Missing advice on M-x manual-entry"))
    (setq count (1+ count))

    ;; don't reload during testing
    (setq man-completion-index-files nil)


    ;;-------------------------------------------------------------------------
    ;; load

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'man-completion)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; risyky-local-variable-p

    (if (not (fboundp 'risky-local-variable-p))
        (message "skip, no `risky-local-variable-p'")

      (mapcar (lambda (symbol)
                (unless (boundp symbol)
                  (error "oops, no variable: %S" symbol))
                (unless (risky-local-variable-p symbol (symbol-value symbol))
                  (error "%S not `risky-local-variable-p'" symbol))
                (setq count (1+ count)))
              '(man-completion-at-point-functions)))


    ;;-------------------------------------------------------------------------
    ;; man-completion-at-point-functions -- cleanup on unload

    (add-to-list 'load-path (expand-file-name "."))
    (require 'man-completion-otherpackage)

    (setq man-completion-at-point-functions
          '(man-completion-otherpackage-somefunc))
    (unload-feature 'man-completion-otherpackage t)

    (if (not (boundp 'unload-feature-special-hooks))
        (message "skip, no unload-feature-special-hooks")

      (when (memq 'man-completion-otherpackage-somefunc
                  man-completion-at-point-functions)
        (error "man-completion-otherpackage-somefunc not removed from -functions"))
      (setq count (1+ count)))

    (setq man-completion-at-point-functions nil)


    ;;------------------------------------------------------------------------
    ;; man-completion-transform-perl

    (let ((man-completion-cache '(
                                  ;; perl-only
                                  ("constant")
                                  ("3perl constant")

                                  ;; command plus perl
                                  ("sort(1)")
                                  ("1 sort")
                                  ("sort(3perl)")
                                  ("3perl sort")

                                  ;; local 3pm plus perl supplied 3perl
                                  ("autodie")
                                  ("autodie(3perl)")
                                  ("autodie(3pm)")
                                  ("3pm autodie")
                                  ("3perl autodie")

                                  ("already(3pm)")
                                  ("already.pm"))))

      (mapcar (lambda (elem)
                (let* ((str  (nth 1 elem))
                       (want (nth 2 elem)))

                  (let ((got (man-completion-transform-perl str)))
                    (unless (equal want got)
                      (error "man-completion-at-point got %S want %S from %S"
                             got want str)))
                  (setq count (1+ count))))

              '(("constant.pm" "constant(3perl)")
                ("sort.pm"     "sort(3perl)")
                ("autodie.pm"  "autodie(3pm)")
                ("already.pm"  "already.pm")
                )))

    ;;------------------------------------------------------------------------
    ;; man-completion-at-point

    (let ((case-fold-search t))
      (mapcar (lambda (elem)
                (let* ((offset (nth 0 elem))
                       (str    (nth 1 elem))
                       (want   (nth 2 elem))
                       (man-completion-at-point-functions nil))

                  (erase-buffer)
                  (insert str)
                  (goto-char (+ offset (point-min)))
                  (let ((got (man-completion-at-point)))
                    (unless (equal want got)
                      (save-excursion
                        (goto-char (point-max))
                        (insert (format "\n\nwant: %s\ngot:  %s\n" want got)))
                      (error "man-completion-at-point got %S want %S from %S %S"
                             got want offset str)))
                  (setq count (1+ count))))

              '((2 "C<Foo-E<gt>method>" "Foo")
                (2  "S<FooE<lt>Bar>" "Foo")
                (10 "S<FooE<lt>Bar>" "Bar")

                (7 "blah chmod, blah" "chmod")
                (0 "cat(1) man page"  "cat(1)")
                (0 "Foo::Bar"         "Foo::Bar")
                (2 "'.Foo->new" "Foo")
                (3 "CALL<abc"   "CALL")

                (0 "L<perlvar/ENV>"   "perlvar")
                (2 "L<perlvar/ENV>"   "perlvar")

                (0 "/bin/cat foo"  "cat")
                (0 "/sbin/cat foo"  "cat")
                (0 "/usr/bin/cat foo"  "cat")
                (0 "/opt/package/sbin/cat foo"  "cat")
                (0 "/etc/adduser.conf"  "adduser.conf")
                (5 "/etc/logrotate.d/xdm"  "logrotate.d")
                (0 "/etc/logrotate.d/xdm"  nil)

                ;; [ command special case
                (3 "if [ foo = bar ]; then ..."  "[")

                (0 "c"     "c")
                (0 "ct"    "ct")
                (0 "c-t"   "c-t")
                (0 "ca-t"  "ca-t")
                (0 "c-at"  "c-at")
                (0 "c-a-t" "c-a-t")
                (0 "c-aa-t" "c-aa-t")
                (0 "c-\nt" "ct")
                (0 "ca-\nt" "cat")
                (0 "c-\nat" "cat")
                (0 "c-\na-\nt"  "cat")
                (0 "c-\na-\nat" "caat")
                (0 "ca-\na-\nt" "caat")
                (0 "c-\naa-\nt" "caat")
                (14 "--this line--\ncat" "cat")

                ;; hyphenated, point on second part
                (5 "foo-\nbar" "foobar")
                (7 "foo-\nbar" "foobar")

                )))

    ;;------------------------------------------------------------------------
    ;; man-completion-hook-to-list

    (unless (null (man-completion-hook-to-list 't-man-completion--no-such-var))
      (error "man-completion-hook-to-list: no such var should be nil"))
    (setq count (1+ count))

    (setq t-man-completion--hook nil)
    (add-hook 't-man-completion--hook 't-man-completion--noop)
    (unless (equal (man-completion-hook-to-list 't-man-completion--hook)
                   '(t-man-completion--noop))
      (error "man-completion-hook-to-list: list one elem"))
    (setq count (1+ count))
    (with-temp-buffer
      (when (fboundp 'make-local-hook) ;; "obsolete" in emacs
        (make-local-hook 't-man-completion--hook)) ;; for xemacs21
      (add-hook 't-man-completion--hook 't-man-completion--noop2
                t  ;; append
                t) ;; buffer local
      (unless (equal (man-completion-hook-to-list 't-man-completion--hook)
                     '(t-man-completion--noop t-man-completion--noop2))
        (error "man-completion-hook-to-list: local list one + global list one"))
      (setq count (1+ count)))

    (add-hook 't-man-completion--hook 't-man-completion--noop2)
    (unless (equal (man-completion-hook-to-list 't-man-completion--hook)
                   '(t-man-completion--noop2 t-man-completion--noop))
      (error "man-completion-hook-to-list: list two elem"))
    (setq count (1+ count))

    (setq t-man-completion--hook 't-man-completion--noop)
    (unless (equal (man-completion-hook-to-list 't-man-completion--hook)
                   '(t-man-completion--noop))
      (error "man-completion-hook-to-list: single func"))
    (setq count (1+ count))
    (with-temp-buffer
      (when (fboundp 'make-local-hook) ;; "obsolete" in emacs
        (make-local-hook 't-man-completion--hook)) ;; for xemacs21
      (add-hook 't-man-completion--hook 't-man-completion--noop2
                nil ;; prepend
                t)  ;; buffer local
      (unless (equal (man-completion-hook-to-list 't-man-completion--hook)
                     '(t-man-completion--noop2 t-man-completion--noop))
        (error "man-completion-hook-to-list: local list one + global single"))
      (setq count (1+ count)))


    ;;------------------------------------------------------------------------
    ;; man-completion-looking-at-brack

    (mapcar
     (lambda (elem)
       (let* ((want (nth 0 elem))
              (str  (nth 1 elem)))
         (erase-buffer)
         (insert str)
         (mapcar
          (lambda (pos)
            (goto-char (+ pos (point-min)))
            (let ((got (not (not (man-completion-looking-at-brack)))))
              (unless (equal want got)
                (error "man-completion-looking-at-brack prefix got %S want %S"
                       got want))))
          (cddr elem))))

     '((t " [ " 1 2)
       (t "[ " 0 1)
       (t "[" 0 1)
       (t " [" 1 2)
       (nil " [" 0)
       (nil "  [" 0 1)
       (nil "[  " 2 3)
       (nil "[[" 0 1 2)
       (nil " [[ " 0 1 2 3 4)))


    ;;------------------------------------------------------------------------
    ;; man-completion-match-filename

    (mapcar
     (lambda (elem)
       (let* ((want-result (nth 0 elem))
              (want-prefix (nth 1 elem))
              (str         (nth 2 elem))
              (got-result  (not (not (man-completion-match-filename str))))
              (got-prefix  (and got-result
                                (match-string 1 str))))
         (unless (equal want-result got-result)
           (error "man-completion-match-filename result got %S want %S"
                  got-result want-result))
         (unless (equal want-prefix got-prefix)
           (error "man-completion-match-filename prefix got %S want %S"
                  got-prefix want-prefix))))

     '((nil nil   "foo")
       (t   nil   "./foo")
       (t   nil   "/foo")
       (t   "-l " "-l foo")
       (t   "-l " "-l /foo")))


    ;;------------------------------------------------------------------------
    ;; man-completion-handler

    (let ((process-environment (copy-sequence process-environment))
          (default-directory   (expand-file-name default-directory)))
      ;; avoid emacs23 `abbreviate-file-name' putting a "~" in the filename
      ;; completion
      (setenv "HOME" "/")

      (setq man-completion-cache '(("-l t-man-completion.some-page")
                                   ("t-man-completion.some-page")))

      (mapcar (lambda (elem)
                (let* ((str    (nth 0 elem))
                       (pred   (nth 1 elem))
                       (action (nth 2 elem))
                       (want   (nth 3 elem))
                       (got    (man-completion-handler str pred action)))
                  (if (listp got)
                      (setq got (sort got 'string-lessp)))
                  (unless (equal want got)
                    (erase-buffer)
                    (insert "man-completion-handler\n")
                    (insert (format "str    %S\n" str))
                    (insert (format "pred   %S\n" pred))
                    (insert (format "action %S\n" action))
                    (insert "\n")
                    (insert (format "got  %S\n" got))
                    (insert (format "want %S\n" want))
                    (error "man-completion-handler got %S want %S from %S %S %S"
                           got want str pred action))
                  (setq count (1+ count))))
              `(
                ;;-----------------------------------
                ;; try-completion


                ;; -l filename
                ("-l t-man-completion." nil nil
                 ,(concat "-l " (expand-file-name "t-man-completion.el")))

                ;; relative filename
                ;; -l filename, rejected by pred
                ;; only works in emacs22 yet
                ;; ("-l t-man-completion."
                ;;  (lambda (str) (not (string-match "t-man-completion" str)))
                ;;  nil
                ;;  nil)

                ("./t-man-completion." nil nil
                 "./t-man-completion.el")

                ;; absolute filename
                (,(substring (expand-file-name "t-man-completion.el")
                             0 -1)
                 nil nil
                 ,(expand-file-name "t-man-completion.el"))

                ;; page name, not filename
                ("t-man-completion." nil nil
                 "t-man-completion.some-page")

                ;;-----------------------------------
                ;; all-completions

                ;; -l filename
                ("-l t-man-completion." nil t
                 ,(cons (concat "-l " (expand-file-name "t-man-completion.el"))
                        (and (file-exists-p "t-man-completion.el~")
                             (list (concat "-l " (expand-file-name
                                                  "t-man-completion.el~"))))))

                ;; -l filename, rejected by pred
                ("-l t-man-completion."
                 (lambda (str) (not (string-match "t-man-completion" str)))
                 t nil)

                ;;-----------------------------------
                ;; test-completion

                ("-l t-man-completion.nosuch" nil lambda  nil)
                ("-l t-man-completion.el"     nil lambda  t)
                ("./t-man-completion.el"      nil lambda  t)

                )))


    ;;------------------------------------------------------------------------
    ;; man-completion-cache

    (man-completion-reset)
    (let ((lst (man-completion-cache)))
      (unless (null (cdar lst))
        (error "man-completion-cache no descriptions requested, but has them")))
    (setq count (1+ count))

    (let ((lst (man-completion-cache t)))
      (unless (or (null lst) ;; if perhaps the system has no pages at all
                  (stringp (cdar lst)))
        (error "man-completion-cache descriptions requested, but doesn't have them")))
    (setq count (1+ count))


    ;;------------------------------------------------------------------------
    ;; `manual-program' with environ setting

    (if (not my-have-env-on-command-p)
        (message "skip due to no environment setting on command name")

      (man-completion-reset)
      (let ((lst1 (man-completion-cache)))
        (man-completion-reset)
        (let* ((manual-program (concat "MISCBITS_EL_TEST_FOO=bar "
                                       manual-program))
               (lst2 (man-completion-cache)))
          (unless (equal lst1 lst2)
            (error "man-completion-cache didn't work with environ setting in `man-program'")))))

    ;;------------------------------------------------------------------------
    ;; man-completion-completing-help-get

    (let ((man-completion-cache '(("cat"    . "concatenate files")
                                  ("-L cat" . "not a filename")))
          (case-fold-search t))
      (mapcar (lambda (elem)
                (let* ((str (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (man-completion-completing-help-get str)))
                  (unless (equal got want)
                    (error "man-completion-completing-help-get str %S got %S want %S"
                           str got want))
                  (setq count (1+ count))))

              '(("cat" "concatenate files")
                ("./cat" nil)
                ("/cat" nil)
                ("-l cat" nil)
                ("-L cat" "not a filename"))))

    ;;------------------------------------------------------------------------
    ;; man-completion-perl-expand-prefix

    (mapcar (lambda (elem)
              (let* ((name                 (nth 0 elem))
                     (want                 (nth 1 elem))
                     (str                  (nth 2 elem))
                     (man-completion-cache (nth 3 elem))
                     (got                  (man-completion-perl-expand-prefix str)))
                (unless (equal want got)
                  (error "man-completion-perl-expand-prefix %S\n  want %S got %S on cache %S"
                         name want got man-completion-cache))
                (setq count (1+ count))))

            '(("one match"
               "Foo::Bar"
               "Bar" (("X::Y" . nil)
                      ("Foo::Bar" . nil)))

              ("two matches, not unique"
               nil
               "Bar" (("Xyzzy::Bar" . nil)
                      ("Foo::Bar" . nil)))
              ("two matches, not unique - B"
               nil
               "Bar" (("ZZZ::YYY::Bar" . nil)
                      ("Foo::Bar" . nil)))

              ("three matches, not unique"
               nil
               "Bar" (("Bar" . nil)
                      ("Xyzzy::Bar" . nil)
                      ("Foo::Bar" . nil)))

              ("case sensitive, one match"
               "Xyzzy::bar"
               "bar" (("Xyzzy::bar" . nil)
                      ("Foo::BAR" . nil)))
              ))


    ;;------------------------------------------------------------------------
    ;; man-completion-read

    ;; see that `man-completion-read' successfully reaches the
    ;; completing-read call, past the pcm conditionals
    (let ((orig (symbol-function 'completing-read)))
      (unwind-protect
          (progn
            (fset 'completing-read (lambda (prompt collection &rest rest) nil))
            (man-completion-read))
        (fset 'completing-read orig)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'man-completion)
            (prop    '(man-completion-default thing-at-point)))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))
        (unless (apply 'get prop)
          (error "oops, didn't find thing-at-point property"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))

        (let ((func (apply 'get prop)))
          (when func
            (error "unload-feature leftover %S property: %S" prop func)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'man-completion))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count)))

    ;;------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
