;; Copyright 2010, 2012, 2016, 2017 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search nil))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'sh-autoconf)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; sh-mode in a non-file buffer

    (with-temp-buffer
      (sh-mode))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; info-lookup-alist addition

    (require 'info-look)
    (let ((lst (info-lookup->other-modes 'symbol 'sh-mode)))
      (if (not lst)
          (message "info-look.el has no sh-mode, skip")
        (unless (memq 'autoconf-mode lst)
          (error "info-lookup-alist sh-mode other-modes doesn't have autoconf-mode")))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'sh-autoconf))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'sh-autoconf))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

