;; Not sure if this works, quite possibly not.



;;; sb-yahoo-finance-au.el --- shimbun backend for au.finance.yahoo.com

;; Copyright 2005, 2009, 2014, 2016 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 1
;; Keywords: news
;; URL: .......... http://user42.tuxfamily.org/sb-yahoo-finance-au/index.html

;; sb-yahoo-finance-au.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; sb-yahoo-finance-au.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with sb-yahoo-finance-au.el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;; Commentary:

;; This module access various news pages from http://au.finance.yahoo.com.
;; Most are available as RSS as well as web pages, but articles seem to
;; appear on the web pages as much as an hour sooner than the RSS.


(require 'shimbun)

(luna-define-class shimbun-yahoo-finance-au (shimbun) ())

(defvar shimbun-yahoo-finance-au-url "http://au.biz.yahoo.com/financenews/")

(defvar shimbun-yahoo-finance-au-groups-alist
  '(("commodities"    . "commodities.html")
    ("companies"      . "sp100.html")
    ("market-reports" . "market_reports.html")
    ("reuters"        . "reuterfinance_index.html")))

(defvar shimbun-yahoo-finance-au-groups
  (mapcar 'car shimbun-yahoo-finance-au-groups-alist))

(luna-define-method shimbun-index-url ((shimbun shimbun-yahoo-finance-au))
  (concat shimbun-yahoo-finance-au-url
          (cdr (assoc (shimbun-current-group-internal shimbun)
		      shimbun-yahoo-finance-au-groups-alist))))

(luna-define-method shimbun-get-headers ((shimbun shimbun-yahoo-finance-au)
                                         &optional range)
  ;; date/times are for Sydney, which is +1000 normally or +1100 during
  ;; daylight savings; ask the system which is the case currently
  ;; (actually we should ask about each article date/time rather than the
  ;; current time, but let's not worry about that, there's rarely more than
  ;; 2 days of headlines anyway)
  (let ((zone (let ((process-environment (copy-sequence process-environment)))
                (setenv "TZ" "Australia/Sydney")
                (format-time-string "%z")))
        headers)

    (catch 'stop
      (while (re-search-forward "\n<a href=\\([^>]+\\)><B>\\([^>]+\\)</B>" nil t nil)
        (let* ((link    (match-string 1))
               (url     (shimbun-expand-url link shimbun-yahoo-finance-au-url))
               (subject (match-string 2))
               (id      (concat link "@au.biz.yahoo.com"))
               (from    "nobody@au.biz.yahoo.com"))

          ;; Eg: " By Richard Pullin"
          ;;     "<BR> By Paul Marriott"
          ;; or possibly no by-line at all (stop at next article href)
          (save-excursion
            (and (re-search-forward "\n\\(<a href=\\|\\(<BR>\\)? *By \\(.*\\)" nil t)
                 (match-beginning 3)
                 (setq from (concat (match-string 3) "<" from ">"))))

          ;; eg. "Thursday June 2, 2005, 9:45 am</DIV>"
          (re-search-forward "\n[A-Za-z]+ \\([A-Za-z][A-Za-z][A-Za-z]\\)[A-Za-z]* \\([0-9]+\\), \\([0-9]+\\), \\([0-9]+\\):\\([0-9]+\\) \\(am\\|pm\\)</DIV>")
          (let* ((month (cdr (assoc-ignore-case (match-string 1)
                                                timezone-months-assoc)))
                 (day   (string-to-number (match-string 2)))
                 (year  (string-to-number (match-string 3)))
                 (hour  (string-to-number (match-string 4)))
                 (min   (string-to-number (match-string 5)))
                 (time  (progn
                          (if (= 12 hour)
                              (setq hour 0))
                          (if (string-equal "pm" (match-string 6))
                              (setq hour (+ hour 12)))
                          (format "%02d:%02d" hour min)))
                 (date  (shimbun-make-date-string year month day time zone)))

            (when (shimbun-search-id shimbun id)
              (throw 'stop nil))
            (push (shimbun-create-header 0 subject from date id "" 0 0 url)
                  headers)))))
    headers))
